/*  
 * file CcDirectory.java
 *
 * Licensed Materials - Property of IBM and/or HCL
 * Restricted Materials of IBM and/or HCL
 *
 * com.ibm.rational.wvcm.stp.cc.CcDirectory
 *
 * (C) Copyright IBM Corporation 2004, 2015.  All Rights Reserved.
 * (C) Copyright HCL Technologies Ltd. 2017.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp.cc;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import java.util.List;

import javax.wvcm.ControllableFolder;
import javax.wvcm.Feedback;
import javax.wvcm.ResourceList;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

/**
 * <p>
 * A proxy for a directory in a ClearCase view.
 * This directory is either under version control or could potentially be
 * put under version control.
 * </p>
 */
public interface CcDirectory
    extends CcFile, ControllableFolder 
{
    /**
     * Create a new view-private directory at the location specified by this
     * resource.  The request will fail if a resource already exists at that
     * location.
     * @see javax.wvcm.ControllableResource#doCreateResource(Feedback)
     */
    public CcDirectory createCcDirectory(Feedback feedback) throws WvcmException;
    
    /**
     * Create a new VOB hard link in this directory.
     * The request will fail if the directory is not already checked-out
     * or a child with name {@code linkName} already exists.
     * <p>Note: For web views, this method is only supported for targets that
     * are visible in the view at the time of invocation.
     * @param comment Comment to be attached to the newly created link, or {@code null}
     * for no comment.
     * @param targets  Controlled resource to which the new link is created.
     * The resource must exist in the same VOB as this directory.
     * @param linkName Leaf name for the new link to be created in this directory.
     * @param feedback 
     * @return Proxy for the new link object
     * @throws WvcmException
     */
    public CcResource doCreateHardLink(String comment, CcResource target, String linkName, 
            Feedback feedback) throws WvcmException;
    
    /**
     * Create new VOB hard links in this directory.
     * The links will be created with names identical to the leaf of the specified targets.
     * The request will fail if the directory is not already checked-out
     * or a child with name matching one of the target leafs already exists.
     * <p>Note: For web views, this method is only supported for targets that
     * are visible in the view at the time of invocation.
     * @param comment Comment to be attached to the newly created link, or {@code null}
     * for no comment.
     * @param targets  List of controlled resources to which the new links are created.
     * The resources must exist in the same VOB as this directory.
     * @param feedback 
     * @return List of proxies for the new link objects.  If only some of the links can be created,
     * the operation succeeds and only the newly created links are returned from the method.
     * @throws WvcmException
     */
    public ResourceList<CcResource> doCreateHardLink(String comment, ResourceList<CcResource> targets, 
            Feedback feedback) throws WvcmException;

    /**
     * Create a new VOB symbolic link in this directory.
     * The request will fail if the directory is not already checked-out
     * or a child with name {@code linkName} already exists.
     * <p>Note that for a web or snapshot view, it is possible for a new link to be
     * successfully created, but not loaded into the view if the target is not currently
     * visible.
     * @param comment Comment to be attached to the newly created link, or {@code null}
     * for no comment.
     * @param target Path (absolute or relative) to which the new link is created.
     * The path does <strong>not</strong> have to resolve to an existing object. 
     * @param linkName Leaf name for the new link to be created in this directory.
     * @param feedback
     * @return Proxy for the new link object
     * @throws WvcmException
     */
    public CcResource doCreateSymlink(String comment, String target, String linkName, 
            Feedback feedback) throws WvcmException;
    
    /**
     * Create new VOB symbolic links in this directory.
     * The links will be created with names identical to the leaf of the specified targets.
     * The request will fail if the directory is not already checked-out
     * or a child with name matching one of the target leafs  already exists.
     * <p>Note that for a web or snapshot view, it is possible for a new link to be
     * successfully created, but not loaded into the view if the target is not currently
     * visible.
     * @param comment Comment to be attached to the newly created link, or {@code null}
     * for no comment.
     * @param targets List of paths (absolute or relative) to which the new links are created.
     * The paths do not <strong>not</strong> have to resolve to an existing object. 
     * @param feedback
     * @return List of proxies for the new link objects.  If only some of the links can be created,
     * the operation succeeds and only the newly created links are returned from the method.
     * @throws WvcmException
     */
    public ResourceList<CcResource> doCreateSymlink(String comment, List<String> targets, 
            Feedback feedback) throws WvcmException;

    /**
     * The list of String objects that identify the names of version-controlled
     * resources in this CcDirectory that are eclipsed by non-version-controlled resources.
     * <p>
     * A
     * {@link javax.wvcm.Workspace#doUpdate(ResourceList, Feedback) Workspace.doUpdate}
     * or
     * {@link javax.wvcm.Workspace#doMerge(ResourceList, javax.wvcm.Workspace.MergeFlag[], Feedback) Workspace.doMerge}
     * request can give a version-controlled folder a version-controlled bound
     * member that has the same name as an existing uncontrolled bound member.
     * In this case, the uncontrolled bound member takes precedence and is said
     * to &quot;eclipse&quot; the new versioned-controlled bound member.
     */
    PropertyName<List<String>> ECLIPSED_LIST =
      new PropertyName<List<String>>(PROPERTY_NAMESPACE, "eclipsed-list");

    /**
     * Get the {@link #ECLIPSED_LIST} property.
     * @return String list of the names of the eclipsed version-controlled resources
     * @throws WvcmException if this ControllableFolder was not created with
     * {@link #ECLIPSED_LIST} as a wanted property.
     */
    List<String> getEclipsedList() throws WvcmException;


    /**
     * Are there checkouts anywhere in the tree below this directory?
     */
    PropertyName<Boolean> HAS_DESCENDANT_CHECKOUTS =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "has-descendant-checkouts");
    
    /**
     * Get the {@link #HAS_DESCENDANT_CHECKOUTS} property.
     * @return true if there are checkouts anywhere in the tree rooted by this directory, else false.
     * @throws WvcmException if this ControllableFolder was not created with
     * {@link #HAS_DESCENDANT_CHECKOUTS} as a wanted property.
     * */
    boolean getHasDescendantCheckouts() throws WvcmException;

    
    /**
     * Are there hijacks anywhere in the tree below this directory?
     * Not supported for Snapshot view resources.
     */
    PropertyName<Boolean> HAS_DESCENDANT_HIJACKS =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "has-descendant-hijacks");
    
    /**
     * Get the {@link #HAS_DESCENDANT_HIJACKS} property.
     * @return true if there are hijacks anywhere in the tree rooted by this directory, else false.
     * @throws WvcmException if this ControllableFolder was not created with
     * {@link #HAS_DESCENDANT_HIJACKS} as a wanted property.
     * */
    public boolean getHasDescendantHijacks() throws WvcmException;

    /**
     * A list of all checkouts in the tree below this directory. Returns a list of CcFile resources.
     */
    PropertyName<ResourceList<CcFile>> AGGREGATED_CHECKOUT_LIST =
        new PropertyName<ResourceList<CcFile>>(PROPERTY_NAMESPACE, "aggregated-checkout-list");

    /**
     * Get the {@link #AGGREGATED_CHECKOUT_LIST} property.
     * @return List of CcFile proxies representing all checkouts in the tree rooted by this directory.
     * @throws WvcmException if this ControllableFolder was not created with
     * {@link #AGGREGATED_CHECKOUT_LIST} as a wanted property.
     * */
    ResourceList<CcFile> getAggregatedCheckoutList() throws WvcmException;
    
    
    /**
     * A list of all hijacks in the tree below this directory. Returns a list of CcFile resources.
     * Not supported for Snapshot view resources.
     */
    PropertyName<ResourceList<CcFile>> AGGREGATED_HIJACK_LIST =
        new PropertyName<ResourceList<CcFile>>(PROPERTY_NAMESPACE, "aggregated-hijack-list");
    
    /**
     * Get the {@link #AGGREGATED_HIJACK_LIST} property.
     * @return List of CcFile proxies representing all hijacks in the tree rooted by this directory.
     * @throws WvcmException if this ControllableFolder was not created with
     * {@link #AGGREGATED_HIJACK_LIST} as a wanted property.
     * */
    public ResourceList<CcFile> getAggregatedHijackList() throws WvcmException;
    
    /**
     * A list of all the view private files in the tree below this directory.  Does not include
     * checkouts, hijacks or derived objects.  Returns a list of CcFile resources.
     * Only supported for the view root and VOB root directories of automatic views.
     */
    PropertyName<ResourceList<CcFile>> AGGREGATED_VIEW_PRIVATE_LIST = 
        new PropertyName<ResourceList<CcFile>>(PROPERTY_NAMESPACE, "aggregated-view-private-list");
    
    /**
     * Get the {@link #AGGREGATED_VIEW_PRIVATE_LIST} property.
     * @return List of CcFile proxies representing view private files in the tree rooted by this directory.
     * @throws WvcmException if this ControllableFolder was not created with
     * {@link #AGGREGATED_VIEW_PRIVATE_LIST} as a wanted property.
     * */
    public ResourceList<CcFile> getAggregatedViewPrivateList() throws WvcmException;

    /**
     * Is this directory the root of a ClearCase VOB.
     */
    PropertyName<Boolean> IS_VOB_ROOT =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-vob-root");
    
    /**
     * Returns the value of this proxy's {@link #IS_VOB_ROOT} property. 
     * @return true if the directory is the root of a VOB, else false
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    public boolean getIsVobRoot() throws WvcmException;
    
}

